<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:dcworms="DCWormsResSchema.xsd"
	xmlns:debb2dcworms="../../debb2dcworms.xsd"
	xmlns:fun="http://whatever">

	<!-- Specification of the output document -->
	<xsl:output method="xml" version="1.0" encoding="UTF-8"
		indent="yes" />

	<!-- Root of the DCWoRMS XML document -->
	<xsl:variable name="rootDoc" select="/" />

	<!-- Information about schedulers and estimators -->
	<xsl:param name="additionalInformationFileName" select="additionalInformationFileName" />


	<xsl:variable name="nestedDoc"
		select="document($additionalInformationFileName)" />

	<xsl:template match="/">
		<xsl:comment>
			This is DEBB information (taken from PLM XML and DEBBComponent
			files), updated by schedulers and estimators. Moreover, all elements
			are sorted and appear in correct order, according to the schema.
		</xsl:comment>
		<xsl:comment>
			Schedulers and estimators information is loaded from
			<xsl:value-of select="$additionalInformationFileName" />
			file
		</xsl:comment>

		<xsl:apply-templates select="environment" />

	</xsl:template>

	<xsl:template match="environment">
		<xsl:copy>
			<xsl:copy-of select="@*" />
			<xsl:apply-templates select="$nestedDoc//timeEstimationPlugin" />
			<xsl:apply-templates select="resources" />
			<xsl:apply-templates select="templates"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="resources">
		<xsl:copy copy-namespaces="no">
		<xsl:copy-of select="@*" />
			<xsl:apply-templates select="//resources/computingResource" />
			<xsl:element name="scheduler">
				<xsl:attribute name="class">Cluster</xsl:attribute>
				<xsl:attribute name="name">cluster</xsl:attribute>
				<!-- In first version, for simplicity, we assume we have only one scheduler 
					and one scheduling plugin. If $additionalInformationFileName contains more 
					than one, the rest is ignored -->
				<xsl:apply-templates select="$nestedDoc//schedulingPlugin[1]" />
			</xsl:element>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="timeEstimationPlugin">
		<xsl:element name="timeEstimationPlugin">
			<xsl:element name="name">
				<xsl:value-of select="name" />
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template match="schedulingPlugin">
		<xsl:variable name="pluginName" select="name" />
		<xsl:element name="schedulingPlugin">
			<xsl:element name="name">
				<xsl:value-of select="$pluginName" />
			</xsl:element>
			<xsl:if test="frequency != ''">
				<xsl:element name="frequency">
					<xsl:value-of select="frequency" />
				</xsl:element>
			</xsl:if>
		</xsl:element>
	
		<xsl:element name="managedComputingResources">
			<!-- TODO: This parameter should be taken from somewhere. -->
			<xsl:attribute name="include">false</xsl:attribute>

			<!-- Find appropriate resources (identified by class, type and name(s)) 
				and add their names here -->
			<xsl:variable name="resourceClass">
				<xsl:value-of select="resources/class" />
			</xsl:variable>
			<xsl:variable name="resourceType">
				<xsl:value-of select="resources/type" />
			</xsl:variable>

			<!-- Not clear why it does not work this way :/ -->
			<!-- <xsl:variable name="resourceNames"><xsl:value-of select="resources/name"/></xsl:variable> -->
			<xsl:variable name="resourceNames"
				select="$nestedDoc//schedulingPlugin[name=$pluginName]/resources/name" />
			
			<xsl:choose>
				<!-- Resource names are present -->
				<xsl:when test="count($resourceNames) > 0">
					<xsl:for-each select="$resourceNames">
						<xsl:variable name="index" select="position()" />
						<xsl:variable name="resourceName" select="." />
						
						<xsl:apply-templates select="$rootDoc//computingResource[(@name=$resourceName and @class=$resourceClass) or 
							(@name=$resourceName and @class=$resourceClass and @type=$resourceType)]" mode="addManagedResorceName"/>
					</xsl:for-each>
				</xsl:when>
				<!-- Resource names are NOT present -->					
				<xsl:otherwise>
					<xsl:apply-templates select="$rootDoc//computingResource[(@class=$resourceClass) or 
							(@class=$resourceClass and @type=$resourceType)]" mode="addManagedResorceName"/>				
				</xsl:otherwise>
			</xsl:choose>	
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="energyEstimationPlugin">
		<xsl:element name="energyEstimationPlugin">
			<xsl:element name="name">
				<xsl:value-of select="name" />
			</xsl:element>
		</xsl:element>
	</xsl:template>


	<xsl:template match="computingResourceTemplate | computingResource">
		<xsl:copy copy-namespaces="no">
			<xsl:copy-of select="@*" />
			<!-- Find nested elements in appropriate order and copy them -->
			
			<!-- templateId -->
			<xsl:apply-templates select="templateId" />
			
			<!-- location -->
			<xsl:apply-templates select="location" />
			<!-- parameter -->
			<xsl:apply-templates select="parameter" />
						
			<xsl:variable name="debbFullPath" select="fun:getFullPath(.)"/>
			<xsl:variable name="ancestors" select="ancestor::computingResourceTemplate"/>
				
			
			<xsl:if test="$debbFullPath != '' and name(.) = 'computingResource' and count($ancestors) = 0 and @name">
				<!-- fullPath parameter should be added only for computingResource nodes; for computingResourceTemplate it is not needed;
				for computingResource nodes embedded in computingResourceTemplate nodes fullPath is not needed either -->
				<xsl:element name="parameter">
					<xsl:attribute name="name">fullPath</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="$debbFullPath" />
					</xsl:element>
				</xsl:element>
			</xsl:if>				
			
			
			<xsl:variable name="resourceClass" select="@class"/>
			<xsl:variable name="resourceType" select="@type"/>
			<xsl:variable name="resourceName" select="@name"/>
			
			<xsl:if test="not(templateId)">
				<!-- Don't add energyEstimationPlugin information if there is a template defined for the computingResource.
				 It will appear in the template. -->
				<xsl:variable name="estimationPluginName" select="fun:getEnergyEstimator($resourceClass, $resourceType, $resourceName)"/>
				<xsl:variable name="hasProfile" select="profile != ''"/>
	
				<xsl:if test="$hasProfile = false() and $estimationPluginName != ''">
					<xsl:element name="profile">
						<xsl:element name="powerProfile">
							<xsl:element name="energyEstimationPlugin">
								<xsl:element name="name">
									<xsl:value-of select="$estimationPluginName"/>
								</xsl:element>
							</xsl:element>
						</xsl:element>
					</xsl:element>
				</xsl:if>
			</xsl:if>
			
			<!-- profile -->
			<xsl:apply-templates select="profile" />
			<!-- device -->
			<xsl:apply-templates select="device" />
			<!-- resourceUnit -->
			<xsl:apply-templates select="resourceUnit" />
			
			<!-- computingResource -->
			<xsl:apply-templates select="computingResource" />
						
		</xsl:copy>
	</xsl:template>
	
	
	<xsl:template match="device">
		<xsl:copy copy-namespaces="no">
			<xsl:copy-of select="@*" />
			<!-- Find nested elements in appropriate order and copy them -->
			
			<!-- templateId -->
			<xsl:apply-templates select="templateId" />
			
			<!-- location -->
			<xsl:apply-templates select="location" />
			<!-- parameter -->
			<xsl:apply-templates select="parameter" />
						
								
			<xsl:variable name="debbFullPath" select="fun:getFullPath(.)"/>
				
			
			<xsl:if test="$debbFullPath != '' and name(.) = 'device' and @name">
				<!-- fullPath parameter should be added only for computingResource nodes; for computingResourceTemplate it is not needed;
				for computingResource nodes embedded in computingResourceTemplate nodes fullPath is not needed either -->
				<xsl:element name="parameter">
					<xsl:attribute name="name">fullPath</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="$debbFullPath" />
					</xsl:element>
				</xsl:element>
			</xsl:if>	
			
			<xsl:variable name="resourceClass" select="@class"/>
			<xsl:variable name="resourceType" select="@type"/>
			<xsl:variable name="resourceName" select="@name"/>
			
			<xsl:if test="not(templateId)">
				<!-- Don't add energyEstimationPlugin information if there is a template defined for the computingResource.
				 It will appear in the template. -->
				<xsl:variable name="estimationPluginName" select="fun:getEnergyEstimator($resourceClass, $resourceType, $resourceName)"/>
				<xsl:variable name="hasProfile" select="profile != ''"/>
				<xsl:variable name="hasPowerProfile" select="profile/powerProfile != ''"/>
	
				<xsl:if test="$hasProfile = false() and $estimationPluginName != ''">
					<xsl:element name="profile">
						<xsl:element name="powerProfile">
							<xsl:element name="energyEstimationPlugin">
								<xsl:element name="name">
									<xsl:value-of select="$estimationPluginName"/>
								</xsl:element>
							</xsl:element>
						</xsl:element>
					</xsl:element>
				</xsl:if>
			</xsl:if>
			
			<!-- profile -->
			<xsl:apply-templates select="profile" />
						
		</xsl:copy>
	</xsl:template>
	

	<xsl:function name="fun:getEnergyEstimator">
		<xsl:param name="computingResourceClass"/>
		<xsl:param name="computingResourceType"/>
		<xsl:param name="computingResourceName"/>
		
   
		<xsl:variable name="estimatorName" >
			<xsl:choose>
				<xsl:when test="$nestedDoc//energyEstimationPlugin[resources/class=$computingResourceClass and resources/name=$computingResourceName]/name !=''">
					<xsl:value-of select="$nestedDoc//energyEstimationPlugin[resources/class=$computingResourceClass and resources/name=$computingResourceName]/name"/>
				</xsl:when>
				<xsl:when test="$nestedDoc//energyEstimationPlugin[resources/class=$computingResourceClass]/name !=''">
					<xsl:value-of  select="$nestedDoc//energyEstimationPlugin[resources/class=$computingResourceClass ]/name"/>
				</xsl:when>
				<xsl:otherwise>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="$estimatorName"/>
	</xsl:function>
	
	<xsl:template match="computingResource" mode="addManagedResorceName">
		<xsl:variable name="debbFullPath" select="fun:getFullPath(.)"/>
		<xsl:choose>
			<xsl:when test="$debbFullPath != '' and @name">
				<xsl:element name="resourceName">
					<xsl:value-of select="$debbFullPath"></xsl:value-of>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="resourceName">
					<xsl:value-of select="@name"></xsl:value-of>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:function name="fun:getFullPath">
		<xsl:param name="currentComputingResource"/>
		<xsl:variable name="ancestors" select="$currentComputingResource/ancestor::computingResource/@name"/>

		
		<xsl:variable name="fullPath">
			<xsl:for-each select="1 to count($ancestors)">
				<xsl:variable name="index" select="." />
				<xsl:value-of select="$ancestors[$index]"/>
				<xsl:text>/</xsl:text>
			</xsl:for-each>	
			<xsl:value-of select="$currentComputingResource/@name"></xsl:value-of>	
		</xsl:variable>
		
		<xsl:value-of select="$fullPath"/>
	</xsl:function>
	
	<xsl:template match="resourceUnit | parameter | location">
		<xsl:copy-of select="." copy-namespaces="no" />
	</xsl:template>

	<xsl:template match="profile">
		<xsl:copy copy-namespaces="no">
			<xsl:copy-of select="@*" />
			
			
			<xsl:variable name="resourceClass" select="../@class"/>
			<xsl:variable name="resourceType" select="../@type"/>
			<xsl:variable name="resourceName" select="../@name"/>
			

			<!-- Don't add energyEstimationPlugin information if there is a template defined for the computingResource.
			 It will appear in the template. -->
			<xsl:variable name="estimationPluginName" select="fun:getEnergyEstimator($resourceClass, $resourceType, $resourceName)"/>
			<xsl:variable name="hasPowerProfile" select="powerProfile != ''"/>

			<xsl:if test="$hasPowerProfile = false() and $estimationPluginName != ''">
					<xsl:element name="powerProfile">
						<xsl:element name="energyEstimationPlugin">
							<xsl:element name="name">
								<xsl:value-of select="$estimationPluginName"/>
							</xsl:element>
						</xsl:element>
					</xsl:element>
			</xsl:if>
			
			<xsl:apply-templates select="powerProfile" />
			<xsl:apply-templates select="airThroughputProfile" />

		</xsl:copy>
	</xsl:template>

	<xsl:template match="powerProfile">
		<xsl:copy copy-namespaces="no">
			<xsl:variable name="resourceClass" select="../../@class"></xsl:variable>
			<xsl:variable name="resourceType" select="../../@type"></xsl:variable>
			<xsl:variable name="resourceNames" select="../../@name"></xsl:variable>

			<xsl:choose>
				<!-- Resource names are present -->
				<xsl:when test="count($resourceNames) > 0">
					<xsl:for-each select="$resourceNames">
						<xsl:variable name="index" select="position()" />
						<xsl:variable name="resourceName" select="." />
						<xsl:choose>
							<!-- Resource type is present -->
							<xsl:when test="$resourceType != ''">
								<xsl:apply-templates
									select="$nestedDoc//energyEstimationPlugin[resources/class=$resourceClass and resources/type=$resourceType and resources/name=$resourceName]" />
							</xsl:when>
							<xsl:otherwise>
								<xsl:apply-templates
									select="$nestedDoc//energyEstimationPlugin[resources/class=$resourceClass and resources/name=$resourceName]" />
							</xsl:otherwise>
						</xsl:choose>
					</xsl:for-each>
				</xsl:when>
				<!-- No resource names -->
				<xsl:otherwise>
					<xsl:choose>
						<!-- Resource type is present -->
						<xsl:when test="$resourceType != ''">
							<xsl:apply-templates
								select="$nestedDoc//energyEstimationPlugin[resources/class=$resourceClass and resources/type=$resourceType]" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates
								select="$nestedDoc//energyEstimationPlugin[resources/class=$resourceClass]" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
			
			<!-- Copy all children of powerProfile -->
			<xsl:copy-of select="@*|node()" />
		</xsl:copy>

	</xsl:template>

	<xsl:template match="airThroughputProfile">
		<xsl:copy copy-namespaces="no">
			<!-- Copy all children of airThroughputProfile -->
			<xsl:copy-of select="@*|node()" />
		</xsl:copy>		
	</xsl:template>

	<xsl:template match="templates">
		<xsl:copy copy-namespaces="no">
			<xsl:apply-templates/>	
		</xsl:copy>
		
	</xsl:template>
	
	<xsl:template match="@*|node()">
		<xsl:copy>
			<xsl:apply-templates select="@*" />
			<xsl:apply-templates />
		</xsl:copy>
	</xsl:template>

</xsl:stylesheet>